/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
*/

package org.xaffect.components.sink.dataviewer.resources.chartviewer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.custom.StyleRange;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.graphics.Color;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.layout.RowLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.wb.swt.SWTResourceManager;

/**
 * SWT-{@link Composite} displaying information about data displayed by a
 * {@link ChartDataViewer}.
 */
public class Caption extends Composite
{
	Composite generalInfo;
	Composite channels;

	Color bgColor = SWTResourceManager.getColor(SWT.COLOR_INFO_BACKGROUND);

	/**
	 * Creates a new {@link Caption}.
	 * 
	 * @param parent
	 *            {@link Composite} in which the {@link Caption} is displayed.
	 */
	public Caption(Composite parent)
	{
		super(parent, SWT.NONE);
		this.setLayout(new FillLayout());

		this.generateWidgets();
	}

	/**
	 * Generates all needed Widgets.
	 */
	private void generateWidgets()
	{
		SashForm sash = new SashForm(this, SWT.HORIZONTAL);

		this.generalInfo = new Composite(sash, SWT.V_SCROLL);
		GridLayout layout = new GridLayout();
		layout.numColumns = 2;
		this.generalInfo.setLayout(layout);

		this.channels = new Composite(sash, SWT.V_SCROLL);
		this.channels.setLayout(new RowLayout(SWT.VERTICAL));
		this.channels.setBackground(this.bgColor);
	}

	/**
	 * Adds a data-channel to the {[@link Caption}.
	 * 
	 * @param name
	 *            name of the data-channel to be added.
	 * @param color
	 *            color representing the added data-channel.
	 */
	public void addChannel(String name, Color color)
	{
		StyledText txtChannel = new StyledText(this.channels, SWT.SINGLE);
		txtChannel.setEditable(false);
		txtChannel.setText(name);
		StyleRange style = new StyleRange();
		style.start = 0;
		style.length = txtChannel.getText().length();
		style.foreground = color;
		style.background = this.bgColor;
		txtChannel.setStyleRange(style);
		txtChannel.setBackground(this.bgColor);
	}

	/**
	 * Adds a general information about data to the {@link Caption}.
	 * 
	 * @param name
	 *            name describing the kind of information.
	 * @param value
	 *            the actual information about the described data.
	 */
	public void addInfo(String name, String value)
	{
		Label lblName = new Label(this.generalInfo, SWT.NONE);
		lblName.setBackground(this.generalInfo.getBackground());
		if (name != null)
		{
			lblName.setText(name);
		}
		Text txtValue = new Text(this.generalInfo, SWT.NONE);
		if (value != null)
		{
			txtValue.setText(value);
		}
		txtValue.setEditable(false);
	}

	/**
	 * Sets the background-{@link Color} of the channel-info-panel of the
	 * {@link Caption}.
	 * 
	 * @param color
	 *            {@link Color} the channel-info-panel is supposed to have.
	 */
	public void setCaptionColor(Color color)
	{
		this.bgColor = color;
		this.channels.setBackground(color);
	}
}
