/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */

package org.xaffect.components.sink.dataviewer.resources.chartviewer;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.xaffect.components.sink.dataviewer.resources.common.Formatter;

/**
 * Represents a composite displaying information about elapsed time and
 * time-span
 */
public class InfoBar extends Composite
{
	Display display;
	private Label lblTime;
	private Label lblTimeSpan;
	private long currentTimeInMs = 0L;

	/**
	 * Creates a new {@link InfoBar}.
	 * 
	 * @param parent
	 */
	public InfoBar(Composite parent)
	{
		super(parent, SWT.NONE);
		this.generateWidgets();
		this.display = parent.getDisplay();
	}

	/**
	 * Generates all needed widgets.
	 */
	private void generateWidgets()
	{
		this.setLayout(new FormLayout());

		this.lblTime = new Label(this, SWT.NONE);
		this.lblTime.setText("Time: 0 hrs 0' 0\"");
		FormData fd_lblTime = new FormData();
		fd_lblTime.left = new FormAttachment(0, 2);
		fd_lblTime.top = new FormAttachment(0, 2);
		fd_lblTime.width = 150;
		this.lblTime.setLayoutData(fd_lblTime);

		this.lblTimeSpan = new Label(this, SWT.NONE);
		this.lblTimeSpan.setAlignment(SWT.CENTER);
		this.lblTimeSpan.setText("<-- " + 0.0 + " sec -->");
		FormData fd_lblTimeSpan = new FormData();
		fd_lblTimeSpan.top = new FormAttachment(this.lblTime, 0, SWT.TOP);
		fd_lblTimeSpan.left = new FormAttachment(this.lblTime, 5, SWT.RIGHT);
		fd_lblTimeSpan.right = new FormAttachment(100, -2);
		this.lblTimeSpan.setLayoutData(fd_lblTimeSpan);
	}

	/**
	 * Updates the information about elapsed time.
	 * 
	 * @param milliseconds
	 *            to be displayed.
	 */
	public void updateTimeLabel(long milliseconds)
	{
		if (InfoBar.this.lblTime != null && !InfoBar.this.lblTime.isDisposed()
				&& milliseconds - this.currentTimeInMs > 1000)
		{
			this.currentTimeInMs = milliseconds;
			InfoBar.this.lblTime.setText(Formatter.formatTime(this.currentTimeInMs));
		}
	}

	/**
	 * Updates the information about the displayed time-span.
	 * 
	 * @param milliseconds
	 *            to be displayed.
	 */
	public void updateTimeSpanLabel(long milliseconds)
	{
		if (InfoBar.this.lblTimeSpan != null && !InfoBar.this.lblTimeSpan.isDisposed())
		{
			InfoBar.this.lblTimeSpan.setText("<-- " + Double.toString(milliseconds / 1000.0) + " sec -->");
		}
	}
}
