/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org [^]

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/. [^]
*/ 

package org.xaffect.components.sink.dataviewer.resources.common;

import org.eclipse.swt.SWT;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.widgets.Menu;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;

/**
 * Represents a drop-down-menu as a {@link ToolItem}. Can be placed in a {@link ToolBar}.
 */
public class DropDownMenu extends Menu
{
	ToolItem button;
	ToolBar parent;
	
	/**
	 * Creates a new {@link DropDownMenu}.
	 * @param parent {@link ToolBar} to be placed in.
	 * @param text for the menu.
	 * @param toolTip for the menu.
	 */
	public DropDownMenu(ToolBar parent, String text, String toolTip)
	{
		// generate menu and add it to optionPanel
		super(parent);
		this.parent = parent;

		// generate button to open menu
		this.button = new ToolItem(parent, SWT.PUSH);
		this.button.setText(text);
		this.button.setToolTipText(toolTip);
		this.button.addSelectionListener(new SelectionListener()
		{
			@Override
			public void widgetSelected(SelectionEvent event)
			{
				Point location = DropDownMenu.this.parent.toDisplay(new Point(0,
						DropDownMenu.this.parent.getSize().y));
				DropDownMenu.this.setLocation(location);
				DropDownMenu.this.setVisible(true);
			}

			@Override
			public void widgetDefaultSelected(SelectionEvent event)
			{
				// nothing for now
			}
		});
	}
	
	/**
	 * Returns parent {@link ToolBar}.
	 */
	public ToolBar getParentOptionPanel()
	{
		return this.parent;
	}
	
	/**
	 * Returns the {@link DropDownMenu}'s text.
	 */
	public String getText()
	{
		return this.button.getText();
	}
	
	/**
	 * Sets the {@link DropDownMenu}'s text
	 * @param title to be set.
	 */
	public void setText(String text)
	{
		this.button.setText(text);
	}
	
	@Override
	public void checkSubclass()
	{
		//to enable subclassing
	}
}
