/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org [^]

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/. [^]
 */

package org.xaffect.components.sink.dataviewer.resources.common;

import java.util.concurrent.TimeUnit;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Color;
import org.eclipse.wb.swt.SWTResourceManager;

/**
 * Utility-class to format any kind of data so it can be displayed in a unified
 * way all over the {@link SinkAdvancedDataViewer}
 */
public class Formatter
{
	/**
	 * Formats time in milliseconds to a String.<br />
	 * Time is displayed in the following matter: H hrs MM' SS"
	 * 
	 * @param milliseconds
	 *            timestamp to be displayed
	 * @return String displaying timestamp
	 */
	public static String formatTime(long milliseconds)
	{
		return String.format(
				"%02d hrs %02d' %02d\"",
				TimeUnit.MILLISECONDS.toHours(milliseconds),
				TimeUnit.HOURS.toMinutes(TimeUnit.MILLISECONDS.toHours(milliseconds))
						- TimeUnit.MILLISECONDS.toMinutes(milliseconds), TimeUnit.MILLISECONDS.toSeconds(milliseconds)
						- TimeUnit.MINUTES.toSeconds(TimeUnit.MILLISECONDS.toMinutes(milliseconds)));
	}

	/**
	 * Returns one of 10 SWT-standard-colors depending on given number.
	 */
	public static Color getColor(int number)
	{
		int id = number % 10;
		switch (id)
		{
			case 0:
				return SWTResourceManager.getColor(SWT.COLOR_YELLOW);
			case 1:
				return SWTResourceManager.getColor(SWT.COLOR_CYAN);
			case 2:
				return SWTResourceManager.getColor(SWT.COLOR_GREEN);
			case 3:
				return SWTResourceManager.getColor(SWT.COLOR_MAGENTA);
			case 4:
				return SWTResourceManager.getColor(SWT.COLOR_RED);
			case 5:
				return SWTResourceManager.getColor(SWT.COLOR_DARK_YELLOW);
			case 6:
				return SWTResourceManager.getColor(SWT.COLOR_DARK_CYAN);
			case 7:
				return SWTResourceManager.getColor(SWT.COLOR_DARK_GREEN);
			case 8:
				return SWTResourceManager.getColor(SWT.COLOR_DARK_MAGENTA);
			case 9:
				return SWTResourceManager.getColor(SWT.COLOR_DARK_RED);
			default:
				return SWTResourceManager.getColor(SWT.COLOR_BLACK);
		}
	}
}
