/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */
package org.xaffect.control.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Listener;

/**
 * Contains four buttons that regulate xAffect's application-flow: start
 * Training, start Measuring, stop, reset.<br />
 * the {@link Composite} also displays the passed time of TRAINING- and
 * MEASURING-state.
 * 
 * @author platzek
 */
public class CompositeControl extends Composite
{
	// widgets
	private Group grpControl;
	private Button btnStartTraining;
	private Button btnStartMeasuring;
	private Button btnStop;
	private Button btnNextSubject;
	private Label lblTrainingTime;
	private Label lblMeasuringTime;

	// data
	private MeasurementTimer trainingTimer;
	private MeasurementTimer measuringTimer;

	/**
	 * Creates a new {@link CompositeControl}.
	 * 
	 * @see Composite
	 * @param parent
	 *            see {@link Composite}
	 * @param style
	 *            see {@link Composite}
	 */
	public CompositeControl(Composite parent, int style)
	{
		super(parent, style);

		this.trainingTimer = new MeasurementTimer();
		this.measuringTimer = new MeasurementTimer();

		this.generateWidgets();
		this.registerListeners();
	}

	/**
	 * generates all widgets needed by the {@link CompositeControl}.
	 */
	private void generateWidgets()
	{
		this.setLayout(new FormLayout());

		this.grpControl = new Group(this, SWT.NONE);
		this.grpControl.setText("Control Experiment");
		this.grpControl.setLayout(new FormLayout());
		FormData fd_grpControl = new FormData();
		fd_grpControl.top = new FormAttachment(0);
		fd_grpControl.left = new FormAttachment(0);
		fd_grpControl.right = new FormAttachment(100);
		fd_grpControl.bottom = new FormAttachment(100);
		this.grpControl.setLayoutData(fd_grpControl);

		this.btnStartTraining = new Button(this.grpControl, SWT.NONE);
		FormData fd_btnStart = new FormData();
		fd_btnStart.width = 100;
		fd_btnStart.top = new FormAttachment(0, 5);
		fd_btnStart.left = new FormAttachment(0, 5);
		this.btnStartTraining.setLayoutData(fd_btnStart);
		this.btnStartTraining.setText("Start Training");
		this.btnStartTraining.setEnabled(false);

		this.btnStartMeasuring = new Button(this.grpControl, SWT.NONE);
		FormData fd_btnStartMeasuring = new FormData();
		fd_btnStartMeasuring.width = 100;
		fd_btnStartMeasuring.left = new FormAttachment(this.btnStartTraining, 5, SWT.RIGHT);
		fd_btnStartMeasuring.top = new FormAttachment(this.btnStartTraining, 0, SWT.TOP);
		this.btnStartMeasuring.setLayoutData(fd_btnStartMeasuring);
		this.btnStartMeasuring.setText("Start Measuring");
		this.btnStartMeasuring.setEnabled(false);

		this.btnStop = new Button(this.grpControl, SWT.NONE);
		FormData fd_btnStop = new FormData();
		fd_btnStop.width = 75;
		fd_btnStop.left = new FormAttachment(this.btnStartMeasuring, 5, SWT.RIGHT);
		fd_btnStop.top = new FormAttachment(this.btnStartTraining, 0, SWT.TOP);
		this.btnStop.setLayoutData(fd_btnStop);
		this.btnStop.setText("Stop");
		this.btnStop.setEnabled(false);

		this.btnNextSubject = new Button(this.grpControl, SWT.NONE);
		FormData fd_btnNextSubject = new FormData();
		fd_btnNextSubject.width = 75;
		fd_btnNextSubject.right = new FormAttachment(100, -5);
		fd_btnNextSubject.top = new FormAttachment(this.btnStartTraining, 0, SWT.TOP);
		this.btnNextSubject.setLayoutData(fd_btnNextSubject);
		this.btnNextSubject.setText("NextSubject");
		this.btnNextSubject.setEnabled(false);

		Label lblTrainingTimeText = new Label(this.grpControl, SWT.NONE);
		FormData fd_lblTrainingTimeText = new FormData();
		fd_lblTrainingTimeText.top = new FormAttachment(this.btnStartTraining, 5);
		fd_lblTrainingTimeText.left = new FormAttachment(this.btnStartTraining, 0, SWT.LEFT);
		lblTrainingTimeText.setLayoutData(fd_lblTrainingTimeText);
		lblTrainingTimeText.setText("Training Time:");

		this.lblTrainingTime = new Label(this.grpControl, SWT.NONE);
		FormData fd_lblTrainingTime = new FormData();
		fd_lblTrainingTime.width = 50;
		fd_lblTrainingTime.top = new FormAttachment(lblTrainingTimeText, 0, SWT.TOP);
		fd_lblTrainingTime.left = new FormAttachment(lblTrainingTimeText, 5, SWT.RIGHT);
		this.lblTrainingTime.setLayoutData(fd_lblTrainingTime);
		this.lblTrainingTime.setText("00:00:00");

		Label lblMeasuringTimeText = new Label(this.grpControl, SWT.NONE);
		FormData fd_lblMeasuringTimeText = new FormData();
		fd_lblMeasuringTimeText.top = new FormAttachment(this.lblTrainingTime, 0, SWT.TOP);
		fd_lblMeasuringTimeText.left = new FormAttachment(this.lblTrainingTime, 5);
		lblMeasuringTimeText.setLayoutData(fd_lblMeasuringTimeText);
		lblMeasuringTimeText.setText("Measuring Time:");

		this.lblMeasuringTime = new Label(this.grpControl, SWT.NONE);
		FormData fd_lblMeasuringTime = new FormData();
		fd_lblMeasuringTime.width = 50;
		fd_lblMeasuringTime.top = new FormAttachment(lblMeasuringTimeText, 0, SWT.TOP);
		fd_lblMeasuringTime.left = new FormAttachment(lblMeasuringTimeText, 5, SWT.RIGHT);
		this.lblMeasuringTime.setLayoutData(fd_lblMeasuringTime);
		this.lblMeasuringTime.setText("00:00:00");
	}

	/**
	 * Registers all needed {@link Listener}s to widgets previously created.
	 */
	private void registerListeners()
	{
		this.btnStartTraining.addListener(SWT.Selection, new Listener()
		{
			@Override
			public void handleEvent(Event evt)
			{
				// start timer
				CompositeControl.this.trainingTimer.start();

				// disable button
				if (CompositeControl.this.btnStartTraining != null
						&& !CompositeControl.this.btnStartTraining.isDisposed())
				{
					CompositeControl.this.btnStartTraining.setEnabled(false);
				}
			}
		});

		this.btnStartMeasuring.addListener(SWT.Selection, new Listener()
		{
			@Override
			public void handleEvent(Event evt)
			{
				// manage timers
				CompositeControl.this.trainingTimer.stop();
				CompositeControl.this.measuringTimer.start();

				// disable button
				if (CompositeControl.this.btnStartMeasuring != null
						&& !CompositeControl.this.btnStartMeasuring.isDisposed())
				{
					CompositeControl.this.btnStartMeasuring.setEnabled(false);
				}
			}
		});

		this.btnStop.addListener(SWT.Selection, new Listener()
		{
			@Override
			public void handleEvent(Event evt)
			{
				// stop timers
				CompositeControl.this.trainingTimer.stop();
				CompositeControl.this.measuringTimer.stop();

				// disable button
				if (CompositeControl.this.btnStop != null && !CompositeControl.this.btnStop.isDisposed())
				{
					CompositeControl.this.btnStop.setEnabled(false);
				}
			}
		});

		this.trainingTimer.addListener(new MeasurementTimeListener()
		{
			@Override
			public void timeChanged(final String time)
			{
				if (!CompositeControl.this.isDisposed() && !CompositeControl.this.getDisplay().isDisposed())
				{
					CompositeControl.this.getDisplay().asyncExec(new Runnable()
					{
						@Override
						public void run()
						{
							if (!CompositeControl.this.isDisposed() && CompositeControl.this.lblTrainingTime != null
									&& !CompositeControl.this.lblTrainingTime.isDisposed())
							{
								CompositeControl.this.lblTrainingTime.setText(time);
							}
						}
					});
				}
			}
		});

		this.measuringTimer.addListener(new MeasurementTimeListener()
		{
			@Override
			public void timeChanged(final String time)
			{
				if (!CompositeControl.this.isDisposed() && !CompositeControl.this.getDisplay().isDisposed())
				{
					CompositeControl.this.getDisplay().asyncExec(new Runnable()
					{
						@Override
						public void run()
						{
							if (CompositeControl.this.lblMeasuringTime != null
									&& !CompositeControl.this.lblMeasuringTime.isDisposed())
							{
								CompositeControl.this.lblMeasuringTime.setText(time);
							}
						}
					});
				}
			}
		});
	}

	/**
	 * Adds a {@link Listener} that will be notified as soon as the
	 * "start Training"-button is pressed.
	 * 
	 * @param listener
	 *            to be added
	 */
	public void addTrainingListener(Listener listener)
	{
		this.btnStartTraining.addListener(SWT.Selection, listener);
	}

	/**
	 * Adds a {@link Listener} that will be notified as soon as the
	 * "start Measuring"-button is pressed.
	 * 
	 * @param listener
	 *            to be added
	 */
	public void addMeasuringListener(Listener listener)
	{
		this.btnStartMeasuring.addListener(SWT.Selection, listener);
	}

	/**
	 * Adds a {@link Listener} that will be notified as soon as the
	 * "stop"-button is pressed.
	 * 
	 * @param listener
	 *            to be added
	 */
	public void addStopListener(Listener listener)
	{
		this.btnStop.addListener(SWT.Selection, listener);
	}

	/**
	 * Adds a {@link Listener} that will be notified as soon as the
	 * "Next Subject"-button is pressed.
	 * 
	 * @param listener
	 *            to be added
	 */
	public void addNextSubjectListener(Listener listener)
	{
		this.btnNextSubject.addListener(SWT.Selection, listener);
	}

	/**
	 * Enables the "start Training"-button.
	 */
	public void enableTraining()
	{
		if (this.btnStartTraining != null && !this.btnStartTraining.isDisposed())
		{
			this.btnStartTraining.setEnabled(true);
		}
	}

	/**
	 * Enables the "start Measuring"-button.
	 */
	public void enableMeasuring()
	{
		if (this.btnStartMeasuring != null && !this.btnStartMeasuring.isDisposed())
		{
			this.btnStartMeasuring.setEnabled(true);
		}
	}

	/**
	 * Enables the "stop"-button.
	 */
	public void enableStopping()
	{
		if (this.btnStop != null && !this.btnStop.isDisposed())
		{
			this.btnStop.setEnabled(true);
		}
	}

	/**
	 * Enables the "Next Subject"-button.
	 */
	public void enableNextSubject()
	{
		if (this.btnNextSubject != null && !this.btnNextSubject.isDisposed())
		{
			this.btnNextSubject.setEnabled(true);
		}
	}

	@Override
	public void setEnabled(boolean enabled)
	{
		if (this.btnStartTraining != null && !this.btnStartTraining.isDisposed() && this.btnStartMeasuring != null
				&& !this.btnStartMeasuring.isDisposed() && this.btnStop != null && !this.btnStop.isDisposed()
				&& this.btnNextSubject != null && !this.btnNextSubject.isDisposed())
		{
			this.btnStartTraining.setEnabled(enabled);
			this.btnStartMeasuring.setEnabled(enabled);
			this.btnStop.setEnabled(enabled);
			this.btnNextSubject.setEnabled(enabled);
		}
	}
}
