/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
 */
package org.xaffect.control.gui;

import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.FormAttachment;
import org.eclipse.swt.layout.FormData;
import org.eclipse.swt.layout.FormLayout;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Group;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.xaffect.desktop.config.ConfigFile;

/**
 * Contains two {@link Text}-fields for SubjectID and SessionID as well as two
 * {@link Combo}-fields for InstitutionID and StudyID.<br />
 * The Component will get the data for both dropdown-menus by itself from
 * xAffect's {@link ConfigFile}.
 * 
 * @author platzek
 */
public class CompositeSubjectInfo extends Composite
{
	// widgets
	private Group grpSubjectInfo;
	private Text textSubjectId;
	private Text textSessionId;
	private Combo comboInstitutionId;
	private Combo comboStudyId;

	// data
	private String[] availableInstitutionIds;
	private String[] availableStudyIds;

	/**
	 * Creates a new {@link CompositeSubjectInfo}
	 * 
	 * @see Composite
	 * @param parent
	 *            see {@link Composite}
	 * @param style
	 *            see {@link Composite}
	 */
	public CompositeSubjectInfo(Composite parent, int style, String[] availableStudyIds,
			String[] availableInstitutionIds)
	{
		super(parent, style);

		if (availableStudyIds != null && availableStudyIds.length > 0)
		{
			this.availableStudyIds = availableStudyIds;
		}
		else
		{
			this.availableStudyIds = new String[] { "none" };
		}

		if (availableInstitutionIds != null && availableInstitutionIds.length > 0)
		{
			this.availableInstitutionIds = availableInstitutionIds;
		}
		else
		{
			this.availableInstitutionIds = new String[] { "none" };
		}

		this.generateWidgets();
		this.updateData();
	}

	/**
	 * generates all widgets that are needed by the {@link CompositeSubjectInfo}
	 */
	private void generateWidgets()
	{
		this.setLayout(new FormLayout());

		this.grpSubjectInfo = new Group(this, SWT.NONE);
		this.grpSubjectInfo.setLayout(new FormLayout());
		FormData fd_grpSubjectInfo = new FormData();
		fd_grpSubjectInfo.top = new FormAttachment(0);
		fd_grpSubjectInfo.left = new FormAttachment(0);
		fd_grpSubjectInfo.right = new FormAttachment(100, 0);
		fd_grpSubjectInfo.bottom = new FormAttachment(100, 0);
		this.grpSubjectInfo.setLayoutData(fd_grpSubjectInfo);
		this.grpSubjectInfo.setText("Subject Information");

		// Subject ID
		Label lblSubjectId = new Label(this.grpSubjectInfo, SWT.NONE);
		FormData fd_lblSubjectId = new FormData();
		fd_lblSubjectId.top = new FormAttachment(0, 10);
		fd_lblSubjectId.left = new FormAttachment(0, 5);
		lblSubjectId.setLayoutData(fd_lblSubjectId);
		lblSubjectId.setText("Subject ID");
		textSubjectId = new Text(this.grpSubjectInfo, SWT.BORDER);
		FormData fd_textSubjectId = new FormData();
		fd_textSubjectId.top = new FormAttachment(lblSubjectId, 5);
		fd_textSubjectId.left = new FormAttachment(lblSubjectId, 0, SWT.LEFT);
		fd_textSubjectId.right = new FormAttachment(25, 0);
		textSubjectId.setLayoutData(fd_textSubjectId);

		// Session ID
		textSessionId = new Text(this.grpSubjectInfo, SWT.BORDER);
		FormData fd_textSessionId = new FormData();
		fd_textSessionId.top = new FormAttachment(this.textSubjectId, 0, SWT.TOP);
		fd_textSessionId.left = new FormAttachment(this.textSubjectId, 5);
		fd_textSessionId.right = new FormAttachment(50, 0);
		textSessionId.setLayoutData(fd_textSessionId);
		Label lblSessionId = new Label(this.grpSubjectInfo, SWT.NONE);
		FormData fd_lblSessionId = new FormData();
		fd_lblSessionId.top = new FormAttachment(lblSubjectId, 0, SWT.TOP);
		fd_lblSessionId.left = new FormAttachment(this.textSessionId, 0, SWT.LEFT);
		lblSessionId.setLayoutData(fd_lblSessionId);
		lblSessionId.setText("Session ID");

		// Institution ID
		this.comboInstitutionId = new Combo(this.grpSubjectInfo, SWT.BORDER | SWT.READ_ONLY);
		FormData fd_comboInstitutionId = new FormData();
		fd_comboInstitutionId.top = new FormAttachment(this.textSubjectId, 0, SWT.TOP);
		fd_comboInstitutionId.left = new FormAttachment(this.textSessionId, 5);
		fd_comboInstitutionId.right = new FormAttachment(75, 0);
		comboInstitutionId.setLayoutData(fd_comboInstitutionId);
		Label lblInstitutionId = new Label(this.grpSubjectInfo, SWT.NONE);
		FormData fd_lblInstitutionId = new FormData();
		fd_lblInstitutionId.top = new FormAttachment(lblSubjectId, 0, SWT.TOP);
		fd_lblInstitutionId.left = new FormAttachment(this.comboInstitutionId, 0, SWT.LEFT);
		lblInstitutionId.setLayoutData(fd_lblInstitutionId);
		lblInstitutionId.setText("Institution ID");

		// Study ID
		this.comboStudyId = new Combo(this.grpSubjectInfo, SWT.READ_ONLY);
		FormData fd_comboStudyId = new FormData();
		fd_comboStudyId.top = new FormAttachment(this.textSubjectId, 0, SWT.TOP);
		fd_comboStudyId.left = new FormAttachment(this.comboInstitutionId, 5);
		fd_comboStudyId.right = new FormAttachment(100, -5);
		comboStudyId.setLayoutData(fd_comboStudyId);
		Label lblStudyId = new Label(this.grpSubjectInfo, SWT.NONE);
		FormData fd_lblStudyId = new FormData();
		fd_lblStudyId.top = new FormAttachment(lblSubjectId, 0, SWT.TOP);
		fd_lblStudyId.left = new FormAttachment(this.comboStudyId, 0, SWT.LEFT);
		lblStudyId.setLayoutData(fd_lblStudyId);
		lblStudyId.setText("Study ID");
	}

	/**
	 * Fetches the data for both {@link Combo}-fields from xAffect's
	 * {@link ConfigFile} and updates the dropdown-menus.
	 */
	private void updateData()
	{
		// fill widgets with data
		this.comboInstitutionId.setItems(this.availableInstitutionIds);
		if (this.availableInstitutionIds != null && this.availableInstitutionIds.length > 0)
		{
			this.comboInstitutionId.setText(this.availableInstitutionIds[0]);
		}
		if (this.availableStudyIds != null && this.availableStudyIds.length > 0)
		{
			this.comboStudyId.setItems(this.availableStudyIds);
			this.comboStudyId.setText(this.availableStudyIds[0]);
		}
	}

	/**
	 * @return the inserted session-id
	 */
	public String getSessionID()
	{
		if (this.textSessionId != null && !this.textSessionId.isDisposed())
		{
			return this.textSessionId.getText();
		}
		else
		{
			return null;
		}
	}

	/**
	 * @return the inserted subject-id
	 */
	public String getSubjectID()
	{
		if (this.textSubjectId != null && !this.textSubjectId.isDisposed())
		{
			return this.textSubjectId.getText();
		}
		else
		{
			return null;
		}
	}

	/**
	 * @return the chosen institution-id
	 */
	public String getInstitutionID()
	{
		if (this.comboInstitutionId != null && !this.comboInstitutionId.isDisposed())
		{
			return this.comboInstitutionId.getText();
		}
		else
		{
			return null;
		}
	}

	/**
	 * @return the chosen study-id
	 */
	public String getStudyID()
	{
		if (this.comboStudyId != null && !this.comboStudyId.isDisposed())
		{
			return this.comboStudyId.getText();
		}
		else
		{
			return null;
		}
	}

	@Override
	public void setEnabled(boolean enabled)
	{
		if (this.textSubjectId != null && !this.textSubjectId.isDisposed() && this.textSessionId != null
				&& !this.textSessionId.isDisposed() && this.comboInstitutionId != null
				&& !this.comboInstitutionId.isDisposed() && this.comboStudyId != null
				&& !this.comboStudyId.isDisposed())
		{
			this.textSubjectId.setEnabled(enabled);
			this.textSessionId.setEnabled(enabled);
			this.comboInstitutionId.setEnabled(enabled);
			this.comboStudyId.setEnabled(enabled);
		}
	}
}
