/*
xAffect - Middleware for Physiological Sensors and Affective Signal Processing
Copyright (c) 2011-2012 FZI Research Center for Information Technology, Germany

This file is part of the xAffect middleware. For more information, see http://www.xaffect.org.

This software is licensed under the Original BSD License. You should have received a copy of the Original BSD License along with xAffect. If not, see http://www.xaffect.org/license/.
*/ 

package org.xaffect.desktop.config;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * Represents the xaffect.properties-file.
 */
public class ConfigFile extends Properties
{

	private static final long serialVersionUID = 1654798765416327469L;
	private Logger logger;

	/**
	 * Creates a new {@link ConfigFile}.
	 */
	public ConfigFile(Logger logger)
	{
		this.logger = logger;
		try
		{
			super.load(new FileInputStream(DesktopConstants.CONFIG_FILE));
		}
		catch (IOException e)
		{
			e.printStackTrace();
			this.logger.log(Level.WARNING, "Cannot read " + DesktopConstants.CONFIG_FILE + ".");
		}
	}

	/**
	 * Gets an array of property-values from property-file. Properties will get
	 * split by ; and stripped of any leading or tailing whitespace.
	 * 
	 * @param identifier
	 *            name of the property to get
	 * @return array of values
	 */
	public String[] getPropertyArray(String identifier)
	{
		String property;
		if (!(property = super.getProperty(identifier)).isEmpty())
		{
			String[] properties = property.split(";");
			for (String prop : properties)
			{
				prop = prop.trim();
			}
			return properties;
		}
		else
		{
			return null;
		}
	}
}
